"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isStyleFile = exports.isShouldSkip = exports.isNotJsLikeFile = exports.getReplacePath = exports.getImportFileDistPath = exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _utils = require("@modern-js/utils");

var _core = require("@babel/core");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let replaceValueHash = {};

const getImportFileDistPath = (importName, currentFilename, opts) => {
  var _opts$sourceDir, _opts$sourceDir2;

  const {
    staticDir = '.',
    styleDir = '.'
  } = opts;

  const currentFileDir = _path.default.dirname(currentFilename); // NB: REL means 'relative'
  // NB: opts.sourceDir is a absolute path


  const crtFileRELSrcPath = _path.default.relative(currentFileDir, (_opts$sourceDir = opts.sourceDir) !== null && _opts$sourceDir !== void 0 ? _opts$sourceDir : currentFileDir);

  const importFileRELSrcPath = _path.default.relative((_opts$sourceDir2 = opts.sourceDir) !== null && _opts$sourceDir2 !== void 0 ? _opts$sourceDir2 : currentFileDir, _path.default.join(currentFileDir, importName));

  const resourceDir = isStyleFile(importName) ? styleDir : staticDir; // crtFileRELSrcPath: Relative path of the current file to the source directory
  // resourceDir: styleDir or staticDir
  // importFileRELPath: Relative path of the import file (eg: index.less or demo.png) to the source directory

  const ret = _path.default.join(crtFileRELSrcPath, resourceDir, importFileRELSrcPath);

  return ret.startsWith('.') ? (0, _utils.slash)(ret) : (0, _utils.slash)(`./${ret}`);
};
/**
 * when the file suffix is not '.js', '.jsx', '.ts', '.tsx', return true;
 * @param file
 * @param filename The filename is the file currently being compiled
 * @returns
 */


exports.getImportFileDistPath = getImportFileDistPath;

const isNotJsLikeFile = (importFileName, currentCompileFile) => {
  const tests = [[/\.js$/, '.js'], [/\.jsx$/, '.jsx'], [/\.ts$/, '.ts'], [/\.tsx$/, '.tsx']]; // check this file is static file
  // by string and determine if the file with the added suffix exists

  return !tests.some(([regex, prefix]) => regex.test(importFileName) || _utils.fs.existsSync(_path.default.join(_path.default.dirname(currentCompileFile), importFileName) + prefix) || _utils.fs.existsSync(_path.default.join(_path.default.dirname(currentCompileFile), importFileName, 'index') + prefix));
};

exports.isNotJsLikeFile = isNotJsLikeFile;

const isStyleFile = file => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some(regex => regex.test(file));
};
/**
 * Skip the following cases:
 * + import 'xxx'
 * + import './a.js'.
 * + import './a' and a is js file
 */


exports.isStyleFile = isStyleFile;

const isShouldSkip = (importName, filename) => {
  if (importName.startsWith('./') || importName.startsWith('../')) {
    if (isNotJsLikeFile(importName, filename)) {
      return false;
    }
  }

  return true;
};

exports.isShouldSkip = isShouldSkip;

const getReplacePath = (opts, importName, currentFilename) => {
  const {
    importStyle
  } = opts;

  if (!currentFilename || !importName) {
    return '';
  }

  if (isShouldSkip(importName, currentFilename)) {
    return '';
  }

  let realFilepath = getImportFileDistPath(importName, currentFilename, opts);

  if (isStyleFile(realFilepath) && importStyle === 'compiled-code') {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, '.css');
  }

  return realFilepath;
};

exports.getReplacePath = getReplacePath;

const importPath = () => ({
  name: 'import-path',

  pre() {
    replaceValueHash = {};
  },

  visitor: {
    Program(nodePath, {
      opts,
      file
    }) {
      nodePath.traverse({
        ImportDeclaration({
          node
        }) {
          const {
            source
          } = node;
          const {
            opts: {
              filename
            }
          } = file;
          const importName = source !== null && source !== void 0 && source.value ? source.value : undefined;
          const replaceValue = getReplacePath(opts, importName, filename); // eslint-disable-next-line @typescript-eslint/restrict-plus-operands

          const hashKey = filename + (importName || '');

          if (replaceValue) {
            if (typeof filename === 'string' && !replaceValueHash[hashKey]) {
              node.source.value = replaceValue;
              replaceValueHash[hashKey] = replaceValue;
            } else if (typeof filename === 'string' && replaceValueHash[hashKey]) {
              node.source.value = replaceValueHash[hashKey];
            } else {
              node.source.value = replaceValue;
            }
          }
        },

        // dynamic import
        CallExpression({
          node
        }) {
          const {
            filename
          } = file.opts;
          const {
            callee,
            arguments: args
          } = node;

          if (callee.type === 'Import' || callee.type === 'Identifier' && callee.name === 'require') {
            const firstArg = args[0];

            if (firstArg.value) {
              const importName = firstArg.value;
              const replaceValue = getReplacePath(opts, importName, filename); // eslint-disable-next-line @typescript-eslint/restrict-plus-operands

              const hashKey = filename + (importName || '');

              if (replaceValue) {
                if (typeof filename === 'string' && !replaceValueHash[hashKey]) {
                  node.arguments = [_core.types.stringLiteral(replaceValue)];
                  replaceValueHash[hashKey] = replaceValue;
                } else if (typeof filename === 'string' && replaceValueHash[hashKey]) {
                  node.arguments = [_core.types.stringLiteral(replaceValueHash[hashKey])];
                } else {
                  node.arguments = [_core.types.stringLiteral(replaceValue)];
                }
              }
            }
          }
        }

      });
    }

  }
});

var _default = importPath;
exports.default = _default;